import pygit2


def get_git_status(repo_path="."):
    """
    Retrieves the status of files in the given Git repository.

    Parameters:
    repo_path (str): The path to the Git repository. Defaults to the current directory.

    Returns:
    dict: A dictionary with keys 'added', 'modified', 'deleted', and 'renamed',
          each containing a list of file paths that match the respective status.
    """

    # Open the repository
    repo = pygit2.Repository(repo_path)

    # Get the current branch reference
    try:
        current_branch = repo.head
    except pygit2.GitError:
        raise pygit2.GitError(f"{repo_path} is not a git repo or does not have a head")

    # Get the main branch reference
    main_branch = repo.branches["main"]

    # Get the commit objects
    current_commit = repo[current_branch.target]
    main_commit = repo[main_branch.target]

    # Get the diff between the current commit and the main branch commit
    diff = repo.diff(main_commit, current_commit)

    # Prepare dictionaries to store file statuses
    status = {
        "added": [],
        "modified": [],
        "deleted": [],
        "moved": [],
    }

    # Process the diff
    for delta in diff.deltas:
        if delta.status == pygit2.GIT_DELTA_ADDED:
            status["added"].append(delta.new_file.path)
        elif delta.status == pygit2.GIT_DELTA_MODIFIED:
            status["modified"].append(delta.new_file.path)
        elif delta.status == pygit2.GIT_DELTA_DELETED:
            status["deleted"].append(delta.old_file.path)
        elif delta.status == pygit2.GIT_DELTA_RENAMED:
            status["moved"].append((delta.old_file.path, delta.new_file.path))

    return {
        "added": status["added"],
        "modified": status["modified"],
        "deleted": status["deleted"],
        "moved": status["moved"],
    }


def print_out_git_changed_files(print_function, repo_path="."):
    status = get_git_status(repo_path=repo_path)
    for entry in status:
        print_function(f"    {entry}:")
        for file in status[entry]:
            print_function(f"    - '{file}'")


def get_current_git_branch():
    """
    Get the current dirs git branch name.

    Returns
    -------
    str
        The name of the current git branch.
    """
    repo = pygit2.Repository(".")
    return repo.head.shorthand



import os

from pydantic import ValidationError
import platformdirs
import pygit2
import yaml

from brassy.templates.settings_template import SettingsTemplate


def get_git_repo_root(path="."):
    """
    Get the root directory of the Git repository containing the given path.

    Parameters
    ----------
    path : str, optional
        A path within the Git repository. Defaults to the current directory.

    Returns
    -------
    str
        Absolute path to the root of the Git repository. This is usually the
        path containing the .git folder.
    """
    return os.path.abspath(os.path.join(pygit2.Repository(path).path, ".."))


def get_project_config_file_path(app_name):
    """
    Retrieve the project-specific configuration file path for the application.

    Parameters
    ----------
    app_name : str
        Name of the application.

    Returns
    -------
    str
        Path to the project's configuration file.
    """
    project_file = f".{app_name}"
    if os.path.isfile(project_file):
        return project_file
    try:
        return os.path.join(get_git_repo_root(), project_file)
    except pygit2.GitError:
        return project_file


def get_user_config_file_path(app_name):
    """
    Retrieve the user-specific configuration file path for the application.

    Parameters
    ----------
    app_name : str
        Name of the application.

    Returns
    -------
    str
        Path to the user's configuration file.
    """
    return os.path.join(platformdirs.user_config_dir(app_name), "user.config")


def get_site_config_file_path(app_name):
    """
    Retrieve the site-specific configuration file path for the application.

    Parameters
    ----------
    app_name : str
        Name of the application.

    Returns
    -------
    str
        Path to the site's configuration file.
    """
    return os.path.join(platformdirs.site_config_dir(app_name), "site.config")


def get_config_files(app_name):
    """
    Get a list of configuration file paths in order of increasing precedence.

    Parameters
    ----------
    app_name : str
        Name of the application.

    Returns
    -------
    list of str
        List of configuration file paths.
    """
    config_files = []
    for f in [
        get_site_config_file_path,
        get_user_config_file_path,
        get_project_config_file_path,
    ]:
        path = f(app_name)
        config_files.append(path)
    return config_files


def create_config_file(config_file):
    """
    Create a configuration file with default settings.

    Parameters
    ----------
    config_file : str
        Path where the configuration file will be created.
    """
    default_settings = SettingsTemplate()
    config_dir = os.path.dirname(config_file)
    if config_dir:
        os.makedirs(config_dir, exist_ok=True)
    with open(config_file, "wt") as f:
        yaml.dump(default_settings.dict(), f)


def read_config_file(config_file, create_file_if_not_exist=False):
    """
    Read and parse a YAML configuration file.

    Parameters
    ----------
    config_file : str
        Path to the configuration file.
    create_file_if_not_exist : bool
        Creates file if it doesn't exist

    Returns
    -------
    dict
        Parsed configuration settings.
    """
    try:
        with open(config_file, "rt") as f:
            return yaml.safe_load(f)
    except FileNotFoundError:
        if not create_file_if_not_exist:
            return SettingsTemplate().dict()
        else:
            create_config_file(config_file)
            return read_config_file(config_file)


def merge_and_validate_config_files(config_files):
    """
    Merge settings from multiple configuration files and validate them.

    Parameters
    ----------
    config_files : list of str
        List of configuration file paths. The order of the files matters.
        Each file overwrites the values of the previous.

    Returns
    -------
    dict
        Merged and validated configuration settings.

    Raises
    ------
    ValidationError
        If any of the settings do not conform to the `Settings` model.
    """
    settings = {}
    for config_file in config_files:
        file_settings = read_config_file(config_file, create_file_if_not_exist=False)
        try:
            SettingsTemplate(**file_settings)
        except ValidationError as e:
            print(f"Failed to validate {config_file}")
            print(repr(e.errors()[0]))
            raise e
        settings.update(file_settings)
    return settings


def get_settings_from_config_files(app_name):
    """
    Retrieve settings from configuration files without environment overrides.

    Parameters
    ----------
    app_name : str
        Name of the application.

    Returns
    -------
    dict
        Configuration settings merged from files.
    """
    return merge_and_validate_config_files(get_config_files(app_name))


def override_dict_with_environmental_variables(input_dict):
    """
    Override dict values with case insensitive environment variables when available.


    Parameters
    ----------
    input_dict : dict
        Original settings dictionary.

    Returns
    -------
    dict
        Updated settings dictionary with environment variable overrides.
    """
    env_vars = dict(os.environ)
    lower_env_vars = {
        key.lower(): {"env_var": key, "value": value} for key, value in env_vars.items()
    }
    for key in input_dict.keys():
        if key.lower() in lower_env_vars:
            override = lower_env_vars[key.lower()]
            # print(
            #    f"Overriding value {key} with environmental "
            #    f"variable {override['env_var']} "
            #    f"with value {override['value']}"
            # )
            input_dict[key] = override["value"]
    return input_dict


def get_settings(app_name):
    """
    Return application settings from config files and environment variables.

    Parameters
    ----------
    app_name : str
        Name of the application.

    Returns
    -------
    Settings
        An instance of the `Settings` model with all configurations applied.

    Raises
    ------
    ValidationError
        If the final settings do not conform to the `Settings` model.
    """
    file_settings = override_dict_with_environmental_variables(
        get_settings_from_config_files(app_name)
    )
    Settings = SettingsTemplate(**file_settings)
    return Settings



"""
This module provides functionality to generate release notes from YAML files.
It reads YAML files, parses their content, and formats the parsed data into release notes in .rst format.
The release notes can be written to an output file.
"""

import argparse
import os
import rich.progress
import yaml
from datetime import datetime

import pygit2

import brassy.utils.settings_manager as settings_manager

Settings = settings_manager.get_settings("brassy")

import brassy.utils.CLI


run_from_CLI = brassy.utils.CLI.run_from_CLI
if __name__ == "__main__":
    run_from_CLI()



import logging
import rich
from rich.logging import RichHandler
from rich.console import Console as rich_console
from rich.prompt import Confirm
from rich.traceback import install as install_rich_tracebacks

logging.captureWarnings(True)


def init_logger(use_rich):
    """
    Initialize and configure the logger.

    Parameters
    ----------
    use_rich : bool
        If True, sets up rich logging else use standard stream logging

    Returns
    -------
    logger : logging.Logger
        The configured logger instance
    """
    logger = logging.getLogger("build_docs")
    if use_rich:
        install_rich_tracebacks()
        logging_handlers = [RichHandler(rich_tracebacks=True)]
    else:
        logging_handlers = [logging.StreamHandler()]

    logging.basicConfig(level=logging.DEBUG, datefmt="[%X]", handlers=logging_handlers)
    logger.debug("Program initialized")
    return logger


def get_rich_opener(no_format=False):
    """
    Returns the appropriate opener function for rich progress bar.

    Args:
        no_format (bool, optional): If True, returns the opener function without any formatting.
            If False, returns the opener function with formatting. Defaults to False.

    Returns:
        function: The opener function for rich progress bar.
    """
    if no_format:
        return rich.progress.Progress().open
    else:
        return rich.progress.open


def setup_console(no_format=False, quiet=False):
    """
    Set up and return the console for printing messages.

    Args:
        no_format (bool, optional): Whether to disable formatting. Defaults to False.
        quiet (bool, optional): Whether to suppress console output. Defaults to False.

    Returns:
        Console: The configured rich console object.
    """
    if not no_format:
        install_rich_tracebacks()
    console = rich_console(quiet=quiet, no_color=(no_format or quiet))
    return console


def get_boolean_prompt_function(format=True):
    if format:
        return Confirm.ask
    else:

        def bool_prompt(question):
            answer = input(question).lower()
            if answer in ["yes", "y", "ye"]:
                return True
            elif answer in ["no", "n", ""]:
                return False
            else:
                print("Please respond with 'yes' or 'no'.")
                return bool_prompt(question)

        return bool_prompt


def setup_messages(format, quiet):
    global open
    global boolean_prompt
    global RichConsole
    global print
    open = get_rich_opener(no_format=not format)
    RichConsole = setup_console(no_format=not format, quiet=quiet)
    print = RichConsole.print if format else print
    boolean_prompt = get_boolean_prompt_function(format=format)


open = get_rich_opener()
RichConsole = setup_console()
print = RichConsole.print
boolean_prompt = get_boolean_prompt_function()



from datetime import datetime

import brassy
from brassy.brassy import Settings


def get_header_footer(rich_open, header_file=None, footer_file=None):
    """
    Adds a header and/or footer to the given content.

    Args:
        content (str): The content to which the header and/or footer will be added.
        rich_open (function): A function used to open files.
        header_file (str, optional): The file containing the header content. Defaults to None.
        footer_file (str, optional): The file containing the footer content. Defaults to None.

    Returns:
        str: The content with the header and/or footer added.
    """

    def getFile(file):
        if not file:
            return None
        with rich_open(file, "r", description=f"Reading {file}") as file:
            return file.read()

    return getFile(header_file), getFile(footer_file)


def find_duplicate_titles(data):
    """
    Check if there are any duplicate titles in dictionaries of lists of dictionaries.

    Args:
        data (dict): A dictionary containing lists of dictionaries with items
         indexed by "title".

    Returns:
        bool: True if there are duplicate "title" values, False otherwise.
    """
    titles = [entry["title"] for category in data for entry in data[category]]
    return not len(set(titles)) == len(titles)


def format_files_changed_entry(detailed, entry):
    files_changed = "::\n\n"
    for change_type in entry["files"]:
        files_changed += "".join(
            [
                f"    {change_type}: {file}\n"
                for file in filter(lambda x: not x == "", entry["files"][change_type])
            ]
        )
    return files_changed


def generate_file_change_section_list_of_strings(
    entry, line, category, title, description
):
    lines = []
    for change_type in entry["files"]:
        if "{file}" in line:
            for file in filter(lambda x: not x == "", entry["files"][change_type]):
                lines.append(
                    line.format(
                        change_type=category.capitalize(),
                        title=title,
                        description=description,
                        file_change=change_type,
                        file=file,
                    )
                )
        else:
            lines.append(
                line.format(
                    change_type=category.capitalize(),
                    title=title,
                    description=description,
                    file_change=change_type,
                )
            )
    return lines


def generate_section_string(
    section_lines, changelog_entries, release_date, version, footer, header
):
    lines = []
    entry_keywords = [
        "{" + k + "}"
        for k in ["title", "description", "file_change", "file", "change_type"]
    ]
    if any([keyword in line for keyword in entry_keywords for line in section_lines]):
        for category, entries in changelog_entries.items():
            for entry in entries:
                for line in section_lines:
                    title = entry["title"].capitalize() or Settings.default_title
                    description = (
                        entry["description"].capitalize()
                        or Settings.default_description
                    )
                    if "{file_change}" in line:
                        lines.extend(
                            generate_file_change_section_list_of_strings(
                                entry, line, category, title, description
                            )
                        )
                    else:
                        lines.append(
                            line.format(
                                change_type=category.capitalize(),
                                title=title,
                                description=description,
                            ),
                        )
    else:
        for line in section_lines:
            lines.append(line)
    for i, line in enumerate(lines):
        lines[i] = line.format(
            prefix_file=header,
            suffix_file=footer,
            release_version=version,
            release_date=release_date,
        )
    return "\n".join(lines)


def format_release_notes(data, version, release_date=None, header=None, footer=None):
    """
    Format the parsed YAML data into release notes in .rst format.

    Parameters
    ----------
    data : dict
        Parsed content of YAML files.
    version : str, optional
        Version number of the release, by default '1.1'.
    release_date : str, optional
        Release date, by default None, which uses today's date.

    Returns
    -------
    str
        Formatted release notes in .rst format.
    """
    if release_date is None:
        release_date = datetime.now().strftime("%Y-%m-%d")

    header = header or ""
    footer = footer or ""

    release_template = Settings.templates.release_template
    formatted_string = ""
    for section in release_template:
        for section_name, lines in section.items():
            formatted_string = (
                formatted_string
                + generate_section_string(
                    lines, data, release_date, version, footer, header
                )
                + "\n"
            )
    return formatted_string.strip()


def build_release_notes(
    input_files_or_folders,
    console,
    rich_open,
    version=None,
    release_date=None,
    header_file=None,
    footer_file=None,
    working_dir=".",
):
    """
    Build release notes from YAML data.

    Parameters
    ----------
    data : dict
        Parsed content of YAML files.
    version : str, optional
        Version number of the release, by default '1.1'.
    release_date : str, optional
        Release date, by default None, which uses today's date.
    header_file : str, optional
        A header file to prepend to the release notes.
    footer_file : str, optional
        A footer file to suffix to the release notes.

    Returns
    -------
    str
        Formatted release notes in .rst format.
    """
    yaml_files = brassy.utils.CLI.get_file_list_from_cli_input(
        input_files_or_folders, console, working_dir=working_dir
    )
    try:
        data = brassy.utils.file_handler.read_yaml_files(yaml_files, rich_open)
    except (ValueError, TypeError) as e:
        console.print(f"[red]{e}")
        exit(1)
    header, footer = get_header_footer(
        rich_open, header_file=header_file, footer_file=footer_file
    )
    content = format_release_notes(
        data, version=version, release_date=release_date, header=header, footer=footer
    )
    return content



import pathlib
from typing import List, Optional, Dict, Union
from datetime import date as Date

import dateparser

from pydantic import (
    BaseModel,
    HttpUrl,
    ValidationError,
    model_validator,
    RootModel,
    Field,
    field_validator,
    validator,
)


class Files(BaseModel):
    deleted: List[str] = []
    moved: List[str] = []
    added: List[str] = []
    modified: List[str] = []

    @model_validator(mode="after")
    def check_at_least_one_field(self):
        if not any(
            getattr(self, field) for field in ["deleted", "moved", "added", "modified"]
        ):
            raise ValueError(
                "At least one of deleted, moved, added, or modified must have a value"
            )
        return self


class RelatedInternalIssue(BaseModel):
    string: Optional[str] = Field(pattern=r"[A-Za-z]+#\d+ - .+", default=None)


class RelatedIssue(BaseModel):
    number: Optional[int] = None
    repo_url: Optional[HttpUrl] = None

    @field_validator("repo_url", mode="before")
    def blank_string(value, field):
        if value == "":
            return None
        return value


class DateRange(BaseModel):
    start: Optional[Date]
    finish: Optional[Date]

    @validator("start", "finish", pre=True, always=True)
    def parse_date(cls, value):
        if value is None or isinstance(value, Date):
            return value
        if isinstance(value, str):
            value = value.strip()
            if not value or value.lower() in ["never", "null"]:
                return None
            try:
                parsed = dateparser.parse(value)
                if parsed is None:
                    raise ValueError(f"Unable to parse date string: {value}")
                return parsed.date()
            except Exception as e:
                raise ValueError(f"Invalid date format: {value}") from e
        raise ValueError(f"Invalid type for date field: {value}")


class ChangeItem(BaseModel):
    title: str
    description: str
    files: Files
    related_issue: Optional[Union[RelatedIssue, RelatedInternalIssue]] = Field(
        alias="related-issue", exclude_unset=True, default=None
    )
    date: Optional[DateRange] = None


class ReleaseNote(RootModel[Dict[str, List[ChangeItem]]]):
    """
    ReleaseNote is a root model containing a dictionary that maps category names to lists of ChangeItems.
    """

    pass


# List of categories stored in a variable
categories = [
    "bug_fix",
    "enhancement",
    "deprecation",
    "removal",
    "performance",
    "documentation",
    "continuous_integration",
]



# This module is for the CLI only portions of brassy.
# Brassy can be run without this file, and importing it should

import argparse
import os

from rich_argparse import RichHelpFormatter
import brassy.actions
import brassy.actions.build_release_notes
import brassy.actions.init
import brassy.actions.prune_yaml
import brassy.utils.file_handler
import brassy.utils.git_handler
from brassy.utils.settings_manager import get_settings
import brassy.utils.messages as messages

Settings = get_settings("brassy")


def get_parser():
    """
    Returns an ArgumentParser object with predefined arguments for generating release notes from YAML files.

    Returns:
        argparse.ArgumentParser: The ArgumentParser object with predefined arguments.
    """
    parser = argparse.ArgumentParser(
        description="Generate release notes from YAML files."
        + " Entries are sorted by order in yaml files, "
        + "and by order of yaml files provided via the command line.",
        formatter_class=RichHelpFormatter,
    )
    parser.add_argument(
        "-t",
        "--write-yaml-template",
        type=str,
        help="Write template YAML to provided file."
        + " If folder provided, place template in folder with current "
        + "git branch name as file name.",
        nargs="?",
        default=argparse.SUPPRESS,
    )
    parser.add_argument(
        "-c",
        "--get-changed-files",
        type=str,
        help="Print git tracked file changes against main."
        + " If directory provided, use that directories checked-out branch.",
        nargs="?",
        default=argparse.SUPPRESS,
    )
    parser.add_argument(
        "input_files_or_folders",
        type=str,
        nargs="*",
        help="The folder(s) containing YAML files and/or YAML files. "
        + "Folders will be searched recursively.",
    )
    parser.add_argument(
        "-r",
        "--release-version",
        type=str,
        default="[UNKNOWN]",
        help="Version number of the release. Default is '[UNKNOWN]'.",
        required=False,
        dest="version",
    )
    parser.add_argument(
        "-d",
        "--release-date",
        type=str,
        help="Date of the release. Default is current system time.",
        required=False,
    )
    parser.add_argument(
        "-nc",
        "--no-color",
        action="store_true",
        default=Settings.use_color,
        help="Disable text formatting for CLI output.",
    )
    parser.add_argument(
        "-p",
        "--prefix-file",
        type=str,
        help="A header file to prepend to the release notes.",
    )
    parser.add_argument(
        "-s",
        "--suffix-file",
        type=str,
        help="A footer file to suffix to the release notes.",
    )
    parser.add_argument(
        "-o", "--output-file", type=str, help="The output file for release notes."
    )
    if Settings.default_yaml_path and Settings.enable_experimental_features:
        yaml_path = os.path.join(".", Settings.default_yaml_path)
    else:
        yaml_path = "."
    parser.add_argument(
        "-yd",
        "--yaml-dir",
        type=str,
        help="Directory to write yaml files to",
        default=yaml_path,
    )
    parser.add_argument(
        "--output-to-console",
        action="store_true",
        default=False,
        help="Write generated release notes to console.",
    )
    parser.add_argument(
        "-nr", "--no-rich", action="store_true", help="Disable rich text output"
    )
    parser.add_argument("-q", "--quiet", action="store_true", help="Only output errors")
    parser.add_argument(
        "-pr", "--prune", action="store_true", help="Prune provided files, do not build"
    )
    parser.add_argument(
        "--init", action="store_true", help="Initialize brassy", default=False
    )
    return parser


def parse_arguments():
    """
    Parse command line arguments for input folder and output file.

    Returns
    -------
    argparse.Namespace
        Parsed arguments containing input_folder and output_file.
    """
    parser = get_parser()
    return parser.parse_args(), parser


def exit_on_invalid_arguments(args, parser, console):
    """
    Validate the argparse arguments.

    This function validates the provided argparse arguments to ensure
    that the required input files/folders and output file are provided.
    If arguments are invalid, it prints an error message and exits the program.

    Parameters
    ----------
    args : argparse.Namespace
        Parsed arguments.

    parser : argparse.ArgumentParser
        The ArgumentParser object used to parse the command-line arguments.
    """
    if bool(args.input_files_or_folders) or "get_changed_files" in args or args.init:
        return

    if "write_yaml_template" in args:
        return

    console.print("[bold red]Invalid arguments.\n")
    parser.print_help()
    exit(1)


def get_yaml_files_from_input(input_files_or_folders):
    """
    Get a list of YAML files from the given input files or folders.

    Parameters
    ----------
    input_files_or_folders : list
        List of paths to input files or folders.

    Returns
    -------
    list
        List of paths to YAML files.

    Raises
    ------
    ValueError
        If a file is not a YAML file or if no YAML files are found in a directory.
    """
    yaml_files = []
    for path in input_files_or_folders:
        if os.path.isfile(path):
            if not path.endswith(".yaml") or path.endswith(".yml"):
                raise ValueError(f"File {path} is not a YAML file.")
            yaml_files.append(path)
        elif os.path.isdir(path):
            dir_yaml_files = []
            for root, dirs, files in os.walk(path):
                for file in files:
                    if file.endswith(".yaml") or file.endswith(".yml"):
                        dir_yaml_files.append(os.path.join(root, file))
            if len(dir_yaml_files) == 0:
                raise FileExistsError(f"No YAML files found in directory {path}.")
            yaml_files.extend(dir_yaml_files)
        else:
            raise FileNotFoundError(path)
    return yaml_files


def get_file_list_from_cli_input(input_files_or_folders, console, working_dir="."):
    try:
        yaml_files = get_yaml_files_from_input(
            [
                brassy.utils.file_handler.get_yaml_template_path(
                    path, working_dir=working_dir
                )
                for path in input_files_or_folders
            ]
        )
    except FileExistsError as e:
        if Settings.fail_on_empty_dir:
            console.print(f"[red]Invalid file or directory: [bold]{e}[/]")
            exit(1)
        else:
            console.print(f"[yellow]Invalid file or directory: [bold]{e}[/]")
            console.print(
                f"[yellow]Returning 0 because fail_on_empty_dir is [bold]False[/]"
            )
            exit(0)
    except FileNotFoundError as e:
        console.print(f"[red]Invalid file or directory: [bold]{e}[/]")
        exit(1)
    except ValueError as e:
        console.print(f"[red]{e}")
        exit(1)
    return yaml_files


def run_from_CLI():
    """
    Main function to generate release notes from YAML files and write to an output file.
    """
    args, parser = parse_arguments()

    messages.setup_messages(format=not args.no_rich, quiet=args.quiet)

    console = messages.RichConsole
    printer = messages.print
    rich_open = messages.open

    exit_on_invalid_arguments(args, parser, console)
    if args.init:
        brassy.actions.init.init()
        exit(0)
    elif args.prune:
        brassy.actions.prune_yaml.direct_pruning_of_files(
            args.input_files_or_folders, console, args.yaml_dir
        )
    elif "write_yaml_template" in args:
        brassy.utils.file_handler.create_blank_template_yaml_file(
            args.write_yaml_template,
            console,
            working_dir=args.yaml_dir,
        )
    elif "get_changed_files" in args:
        brassy.utils.git_handler.print_out_git_changed_files(
            printer,
            repo_path=args.get_changed_files if args.get_changed_files else ".",
        )
    elif args.input_files_or_folders:
        content = brassy.actions.build_release_notes.build_release_notes(
            args.input_files_or_folders,
            console,
            rich_open,
            version=args.version,
            release_date=args.release_date,
            header_file=args.prefix_file,
            footer_file=args.suffix_file,
            working_dir=args.yaml_dir,
        )
        if args.output_file:
            brassy.utils.file_handler.write_output_file(args.output_file, content)
            if not args.quiet:
                console.print(f"[green]Wrote release notes to {args.output_file}")
        else:
            if not args.quiet:
                console.print(
                    f"[green]Release notes built successfully. No output file provided."
                )
        if args.output_to_console:
            console.print(content)

    else:
        parser.print_help()



import pathlib
from typing import List, Optional, Dict

from pydantic import BaseModel, Field


class ReleaseTemplate(BaseModel):
    release_template: Optional[List[Dict[str, List[str]]]] = Field(
        default=None, alias="release-template"
    )

    class Config:
        populate_by_name = True


# Example YAML representation (as a string for reference)
"""
release-template:
  - header:
    - {prefix_file}
  - title:
    - "Version {release_version} ({release_date})"
    - "**************************"
  - summary:
    - " * *{change_type}*: {title}"
  - entry:
    - "{change_type}"
    - "==========="
    - ""
    - "{title}"
    - "-------------------------"
    - ""
    - "{description}"
    - ""
    - "::"
    - ""
    - "     {file_change}: {file}"
  - footer:
    - {suffix_file}
"""

# Corrected instantiation of DefaultTemplate
DefaultTemplate = ReleaseTemplate(
    **{
        "release-template": [
            {"header": ["{prefix_file}", ""]},
            {
                "title": [
                    "",
                    "Version {release_version} ({release_date})",
                    "**************************",
                    "",
                ]
            },
            {"summary": [" * *{change_type}*: {title}"]},
            {
                "entry": [
                    "",
                    "{change_type}",
                    "===========",
                    "",
                    "{title}",
                    "-------------------------",
                    "",
                    "{description}",
                    "",
                    "::",
                    "",
                    "     {file_change}: {file}",
                ]
            },
            {"footer": ["", "{suffix_file}"]},
        ]
    }
)


class SettingsTemplate(BaseModel):
    use_color: bool = True
    default_yaml_path: Optional[pathlib.Path] = None
    change_categories: List[str] = [
        "bug fix",
        "enhancement",
        "deprecation",
        "removal",
        "performance",
        "documentation",
        "continuous integration",
    ]
    default_title: str = "NO TITLE"
    default_description: str = "NO DESCRIPTION"
    fail_on_empty_dir: bool = True

    valid_fields: List[str] = ["title", "description", "files", "related-issue"]
    valid_changes: List[str] = ["deleted", "moved", "added", "modified"]
    enable_experimental_features: bool = False
    templates: Optional[ReleaseTemplate] = DefaultTemplate
